library(readxl)
library(scales) # Package for using percentage
library(IRanges)
library(glue)
library(lubridate) # Timezones
library(magrittr)
library(dplyr) # is needed to use function %>%

# --- Import data ---

# Import raw BG-15v2 data
df_all <- read.csv("0 Raw Results/Platform_test_BG15v2_raw_data.csv")
colnames(df_all) <- c(
  'Timestamp',
  'Air_flow',
  'Gas_flow', 
  'PWM_gasvalve', 
  'SDpump_I', 
  'Inclination', 
  'Steam_flow', 
  'AC_Power',
  'SDtank_LLH', 
  'SDtank_LLL', 
  'Stack_V', 
  'Stack_I', 
  'Burner_T', 
  'Stack_Power', 
  'Elec_eff_net', 
  'Elec_eff_stack', 
  'Stack_in_T', 
  'Stack_out_T', 
  'S2C', 
  'FU', 
  'Gas_flow_req')

parameter_labels <- data.frame(
  Air_flow = 'Air flow [SLM]',
  Gas_flow = 'Fuel flow [SLM]', 
  PWM_gasvalve = 'PWM fuel supply [-]', 
  SDpump_I = 'Current steam dosing pump [A]', 
  Inclination = 'Inclination [°]', 
  Steam_flow = 'Steam flow [SLM]', 
  AC_Power = 'AC power [W]',
  SDtank_LLH = 'LLH steam dosing tank [?]', 
  SDtank_LLL = 'LLL steam dosing tank [?]', 
  Stack_V = 'Stack voltage [V]',
  Stack_I = 'Stack current [A]', 
  Burner_T = 'Burner temp [°C]', 
  Stack_Power = 'Stack power [W]', 
  Elec_eff_net = 'Net elec. efficiency [%]', 
  Elec_eff_stack = 'Elec. efficiency stack [%]', 
  Stack_in_T = 'Stack in temp [°C]', 
  Stack_out_T = 'Stack out temp [°C]', 
  S2C = 'Steam to carbon [-]', 
  FU = 'Fuel utilisation [%]', 
  Gas_flow_req = 'Requested fuel flow [SLM]'
)

df_all$Timestamp <- force_tz(as.POSIXct(df_all$Timestamp, format="%Y-%m-%dT%H:%M:%S.000000Z"), tzone= "UTC", roll=FALSE) # convert from character to POSIXct and define origin of data (UTC +00)
df_all$Timestamp <- with_tz(df_all$Timestamp, "CET") # Change Timesteps from UCT (+00) to CEST (+01)

# Add column with elapsed time
start_time <- df_all[1, 1]
df_all$Elapsed_time <- df_all$Timestamp - start_time
df_all$Elapsed_time <- as.numeric(difftime(df_all$Timestamp, df_all$Timestamp[1], units = "secs"))

# Seperate degredation experiment
begin <- force_tz(as.POSIXct("2023-02-27  11:05:00"), tzone = "CET", roll = FALSE)
end <- force_tz(as.POSIXct("2023-03-10  14:21:00"), tzone = "CET", roll = FALSE) 
df_deg <- df_all %>% filter(Timestamp > begin , Timestamp < end)
df <- df_all %>% subset(Timestamp < begin | Timestamp > end)

# --- Combine test interval, test conditions and measurement data ---
# Import test intervals and test conditions
df_manual <- read_excel("1 Cleaned results/Testconditions_clean.xlsm", range = cell_cols(1:6))
df_manual$Begin_time <- force_tz(df_manual$Begin_time, "CET") # Make sure timezone is also right for manual entries
df_manual$End_time <- force_tz(df_manual$End_time, "CET")
df_manual_deg <- df_manual %>% subset(endsWith(Testphase,'deg') | endsWith(Testphase,'OC11') |  endsWith(Testphase,'OC12'))
df_manual <- df_manual %>% subset(!(endsWith(Testphase,'deg')))

# Exclude rows of test data that are outside the test conditions
# create a list of logical vectors indicating whether each row of df falls within each time interval in df_manual
within_intervals <- lapply(seq_len(nrow(df_manual)), function(i) {
  df$Timestamp >= df_manual$Begin_time[i] & df$Timestamp <= df_manual$End_time[i]
})
# Combine the logical vectors using row-wise OR to determine which rows of df fall within any time interval in df_manual
within_any_interval <- Reduce("|", within_intervals)
# Filter rows in df based on whether they fall within any time interval in df_manual
df_tests <- df[within_any_interval,]

# Add test conditions to dataframe with measurement data
rm(within_intervals,within_any_interval)
gc() # clear now unused memory
joined_df <- merge(df_tests, df_manual, by = NULL, all.x = TRUE)
# Filter rows based on time interval
filtered_df <- joined_df[joined_df$Timestamp >= joined_df$Begin_time &
                           joined_df$Timestamp <= joined_df$End_time, ]
# Remove redundant columns and reorder
df_tests <- filtered_df[, !(names(filtered_df) %in% c("Begin_time", "End_time"))]
df_tests <- df_tests %>% select(c("Timestamp","Testphase","Inclination_set","RPM","Period"), everything())
df_tests$Period <- as.numeric(df_tests$Period)
rm(filtered_df,joined_df)
gc()

# Add colum for load case and rotation direction
df_tests <- df_tests %>%
  mutate(Loadcase = if_else((endsWith(Testphase,'PL50') | endsWith(Testphase,'OC4')| endsWith(Testphase,'OC5')| endsWith(Testphase,'OC9') | endsWith(Testphase,'OC10')), 'Part_load', 'Full_load')) %>%
  mutate(Direction = if_else((grepl('Y',Testphase,fixed=TRUE) | endsWith(Testphase,'OC1') | endsWith(Testphase,'OC2') | endsWith(Testphase,'OC3') | endsWith(Testphase,'OC4') | endsWith(Testphase,'OC5') ),'Y','X') )

# --- Remove unusable or faulty data ---
# Remove 24h gas flow safety check
begin <- force_tz(as.POSIXct("2023-02-13 10:18:58"), tzone = "CET", roll = FALSE)
end <- force_tz(as.POSIXct("2023-02-13 10:21:58"), tzone = "CET", roll = FALSE) 
df_tests <- df_tests %>% subset(Timestamp < begin | Timestamp > end)

begin <- force_tz(as.POSIXct("2023-02-15 10:26:35"), tzone = "CET", roll = FALSE)
end <- force_tz(as.POSIXct("2023-02-15 10:29:18"), tzone = "CET", roll = FALSE) 
df_tests <- df_tests %>% subset(Timestamp < begin | Timestamp > end)

# --- Create separate data frames for different test to improve handability ---
df_static_test <- df_tests[grep("BG15v2_S_",df_tests$Testphase), ]
df_dynamic_test <- df_tests[grep("BG15v2_D",df_tests$Testphase), ]
df_OC <- df_tests[grep("BG15v2_OC",df_tests$Testphase), ]
df_other_test <- df_tests[grep("BG15v2_150RPM|BG15v2_SD|BG15v2_valve|BG15v2_valve2|BG15v2_highRPM",df_tests$Testphase), ]

# --- Add test conditions to degradation dataframe ---
# get a list of vectors where each item is whether the timestamp in df_deg in each interval of df_manual
within_intervals  <- lapply(df_deg$Timestamp, function(z) z >= df_manual_deg$Begin_time & z <= df_manual_deg$End_time)

# combine the two list and use the combined vectors as an index on the test conditions
df_deg$Testphase <- lapply(1:nrow(df_deg), function(i) {
  df_manual_deg$Testphase[ within_intervals[[i]] ]
}) %>% 
  # replace zero length strings with NA values
  sapply( function(df_manual_deg) ifelse(length(df_manual_deg) == 0, 0, df_manual_deg)) 
df_deg['Testphaset'][df_deg['Testphase'] == '-'] <- 0

df_deg$Inclination_set <- lapply(1:nrow(df_deg), function(i) {
  df_manual_deg$Inclination_set[ within_intervals[[i]] ]
}) %>% 
  # replace zero length strings with NA values
  sapply( function(df_manual_deg) ifelse(length(df_manual_deg) == 0, 0, df_manual_deg)) 
df_deg['Inclination_set'][df_deg['Inclination_set'] == '-'] <- 0

df_deg$RPM <- lapply(1:nrow(df_deg), function(i) {
  df_manual_deg$RPM[ within_intervals[[i]] ]
}) %>% 
# replace zero length strings with NA values
sapply( function(df_manual_deg) ifelse(length(df_manual_deg) == 0, 0, df_manual_deg)) 
df_deg['RPM'][df_deg['RPM'] == '-'] <- 0

df_deg$Period <- lapply(1:nrow(df_deg), function(i) {
  df_manual_deg$Period[ within_intervals[[i]] ]
}) %>% 
  # replace zero length strings with NA values
  sapply( function(df_manual_deg) ifelse(length(df_manual_deg) == 0, 0, df_manual_deg)) 
df_deg['Period'][df_deg['Period'] == '-'] <- 0

# reset start time for degradation dataframe
start_time <- df_deg[1, 1]
df_deg$Elapsed_time <- df_deg$Timestamp - start_time
df_deg$Elapsed_time <- as.numeric(difftime(df_deg$Timestamp, df_deg$Timestamp[1], units = "secs"))

rm(within_intervals)

# Create and export dataframe for animation
# Not moving
begin <- force_tz(as.POSIXct("2023-02-23 17:24:16"), tzone = "CET", roll = FALSE)
# Movement start at 2023-02-23 17:24:54
end <- force_tz(as.POSIXct("2023-02-23 17:25:32"), tzone = "CET", roll = FALSE)

df_animate <- df %>%
  dplyr::filter(Timestamp>begin,Timestamp<end)

# add acceleration
z_g <- 1.030/1.899 * 1.109
df_animate$Time_diff
df_animate$Acceleration_t <- c(NA, z_g * (diff(df_animate$Inclination* (pi/180))/diff(df_animate$Elapsed_time)))
  
start_time <- df_animate[1, 1]
df_animate$Elapsed_time <- df_animate$Timestamp - start_time
df_animate$Elapsed_time <- as.numeric(difftime(df_animate$Timestamp, df_animate$Timestamp[1], units = "secs"))








