library(ggplot2)
library(zoo) # For moving average
library(lubridate) # Timezones
library(dplyr) # is needed to use function %>%
library(here) # Package for current file location for saving files
library(GauPro)
library(GPfit)

# ----- Plot degradation test -----
# Timeframe of unplanned shut down

# Made sure there are now water fillings at the beginning or end of this periods for calculating means
# Add row with phase for part before shutdown
begin1 <- force_tz(as.POSIXct("2023-02-27  11:05:00"), tzone = "CET", roll = FALSE)
end1 <- force_tz(as.POSIXct("2023-03-07  09:00:00"), tzone = "CET", roll = FALSE)

# Add row with phase for part during shutdown
begin2 <- force_tz(as.POSIXct("2023-03-06  18:10:00"), tzone = "CET", roll = FALSE)
end2 <- force_tz(as.POSIXct("2023-03-09  09:04:43"), tzone = "CET", roll = FALSE)

# Add row with phase for part after shutdown
begin3 <- force_tz(as.POSIXct("2023-03-09  09:04:43"), tzone = "CET", roll = FALSE)
end3 <- force_tz(as.POSIXct("2023-03-10  14:21:00"), tzone = "CET", roll = FALSE)

begin <- c(begin1,begin2,begin3)
end <- c(end1,end2,end3)
rm(begin1,begin2,begin3,end1,end2,end3)

df_deg <- df_deg %>% 
  mutate(Phase = case_when(
    df_deg$Timestamp >= begin[1] & df_deg$Timestamp <= end[1] ~ "Before",
    df_deg$Timestamp >= begin[2] & df_deg$Timestamp <= end[2] ~ "Shutdown",
    df_deg$Timestamp >= begin[3] & df_deg$Timestamp <= end[3] ~ "After",
    TRUE ~ "other"
  )
  )

# Remove 12h safety check
df_deg_filtered <- df_deg_nofill
check <- which(df_deg_nofill$Air_flow > 180)
for (i in check){
  # Find the range of times to remove
  start_time <- df_deg_nofill[i, "Timestamp"] - 90
  end_time <- df_deg_nofill[i, "Timestamp"] + 150
  # Remove the rows within the specified range
  df_deg_filtered <- df_deg_filtered %>% dplyr::filter(Timestamp < start_time | Timestamp > end_time)
}

rm(check, filling)

# Normalise voltage for current differences
I_start = mean(df_deg_filtered$Stack_I[df_deg_filtered$Testphase=="BG15v2_OC11"])

df_deg_filtered <- df_deg_filtered %>% 
  mutate(Stack_V_comp = Stack_V*(Stack_I/I_start)) %>%
  mutate(R = Stack_V/Stack_I)
rm(I_start)

# Plot the average voltage over time
fs <- 1 # sampling rate (1 second)
fc <- 0.001 # Cut-off frequency (Data with frequencies higher than this will be omitted)
n <- 5

# Compute filter coefficients
b <- butter(n, fc/(fs/2), type = "low")$b
a <- butter(n, fc/(fs/2), type = "low")$a

# Apply zero-phase filtering to signal y
df_deg_filtered$Stack_V_avg <- filtfilt(b, a, df_deg_filtered$Stack_V)
df_deg_filtered$Stack_I_avg <- filtfilt(b, a, df_deg_filtered$Stack_I)
df_deg_filtered$Elec_eff_net_avg <- filtfilt(b, a, df_deg_filtered$Elec_eff_net)

# Calculate degradation for different phases
df_deg_mean <- df_deg_filtered %>%
  group_by(Phase) %>%
  summarize(Time_mean_start = mean(head(Timestamp,300)),                # Mean time of first 5 minutes of Phase (to calculate time difference)
            Time_mean_end = mean(tail(Timestamp,300)),                  # Mean time of last 5 minutes of Phase (to calculate time difference)
            Stack_V_mean_start = mean(head(Stack_V,300)),               # Mean stack voltage of first 5 minutes of Phase (to calculate difference)
            Stack_V_mean_end = mean(tail(Stack_V, 300)),                # Mean stack voltage of last 5 minutes of Phase (to calculate difference)
            Stack_V_comp_mean_start = mean(head(Stack_V_comp,300)),     # Mean compensated stack voltage with current of first 5 minutes of Phase (to calculate difference)
            Stack_V_comp_mean_end = mean(tail(Stack_V_comp, 300)),      # Mean compensated stack voltage with current of last 5 minutes of Phase (to calculate difference)
            R_mean_start = mean(head(R,300)),                           # Mean equivalent resistance of first 5 minutes of Phase (to calculate difference)
            R_mean_end = mean(tail(R, 300)),                            # Mean equivalent resistance of last 5 minutes of Phase (to calculate difference)
            Elec_eff_net_mean_start = mean(head(Elec_eff_net,300)),     # Mean equivalent resistance of first 5 minutes of Phase (to calculate difference)
            Elec_eff_net_mean_end = mean(tail(Elec_eff_net, 300))       # Mean equivalent resistance of last 5 minutes of Phase (to calculate difference)
  )
df_deg_mean <- df_deg_mean %>% add_row(Phase = 'Total', Time_mean_start = df_deg_mean$Time_mean_start[df_deg_mean$Phase == "Before"], Time_mean_end = df_deg_mean$Time_mean_end[df_deg_mean$Phase == "After"], Stack_V_mean_start = df_deg_mean$Stack_V_mean_start[df_deg_mean$Phase == "Before"], Stack_V_mean_end = df_deg_mean$Stack_V_mean_end[df_deg_mean$Phase == "After"], Stack_V_comp_mean_start = df_deg_mean$Stack_V_comp_mean_start[df_deg_mean$Phase == "Before"], Stack_V_comp_mean_end = df_deg_mean$Stack_V_comp_mean_end[df_deg_mean$Phase == "After"], R_mean_start = df_deg_mean$R_mean_start[df_deg_mean$Phase == "Before"], R_mean_end = df_deg_mean$R_mean_end[df_deg_mean$Phase == "After"], Elec_eff_net_mean_start = df_deg_mean$Elec_eff_net_mean_start[df_deg_mean$Phase == "Before"], Elec_eff_net_mean_end = df_deg_mean$Elec_eff_net_mean_end[df_deg_mean$Phase == "After"]) %>%
  # Difference in stack voltage per phase of deg test [V]
  mutate(deg_V = Stack_V_mean_start - Stack_V_mean_end) %>%
  # Voltage degradation per 1000h [V/1000h]
  mutate(deg_V_1000h = 1000*deg_V / as.numeric(difftime(Time_mean_end, Time_mean_start, units="hours"))) %>%
  # Percent voltage degradation per 1000h [V/1000h] (difference in voltage as percentage of Voltage at beginning of degradation test)
  mutate(deg_pV_1000h = label_percent()((deg_V/df_deg_mean$Stack_V_mean_start[df_deg_mean$Phase == "Before"])*(1000/as.numeric(difftime(Time_mean_end, Time_mean_start, units="hours"))))) %>%
  # Compensated percent voltage degradation per 1000h [V/1000h] (difference in voltage as percentage of Voltage at beginning of degradation test)
  mutate(deg_pV_comp_1000h = label_percent()(((Stack_V_comp_mean_start - Stack_V_comp_mean_end) / df_deg_mean$Stack_V_comp_mean_start[df_deg_mean$Phase == "Before"] )*(1000/as.numeric(difftime(Time_mean_end, Time_mean_start, units="hours"))))) %>%
  # Percentage decrease in equivalent resistance
  mutate(deg_pR_1000h = label_percent()(( (R_mean_start - R_mean_end)/ df_deg_mean$R_mean_start[df_deg_mean$Phase == "Before"] )*(1000/as.numeric(difftime(Time_mean_end, Time_mean_start, units="hours"))))) %>%
  # Percentage point decrease in net system efficiency (Absolute)
  mutate(deg_eff_1000h = label_percent(scale = 1,)(( (Elec_eff_net_mean_start - Elec_eff_net_mean_end))*(1000/as.numeric(difftime(Time_mean_end, Time_mean_start, units="hours"))))) %>%
  # Percentage decrease in net system efficiency
  mutate(deg_peff_1000h = label_percent()(( (Elec_eff_net_mean_start - Elec_eff_net_mean_end)/ df_deg_mean$Elec_eff_net_mean_start[df_deg_mean$Phase == "Before"] )*(1000/as.numeric(difftime(Time_mean_end, Time_mean_start, units="hours")))))
df_deg_mean <- df_deg_mean[c(2,3,1,4),]



# Calculate degradation for different phases during inclinations and without 
# Find runs of consecutive Testphase values
r <- rle(df_deg_filtered$Testphase)

# Extract indices of first and last rows in each run
run_starts <- cumsum(r$lengths) - r$lengths + 1
run_ends <- cumsum(r$lengths)

# Calculate means for first and last 300 rows of each run
means_list <- lapply(seq_along(run_starts), function(i) {
  start <- run_starts[i]
  end <- run_ends[i]
  if (end - start >= 599) {
    df_slice <- df_deg_filtered[start:end,]
    means <- lapply(df_slice[, c("Timestamp","Stack_V","Stack_V_comp", "R","Elec_eff_net")], function(x) {
      first_mean <- mean(x[1:300])
      last_mean <- mean(x[(end-start-299): (end-start)])
      return(list(mean_start = first_mean, mean_end = last_mean))
    })
    return(data.frame(Testphase = unique(df_slice$Testphase)[1], Phase = unique(df_slice$Phase)[1], 
                      do.call(c, means)))
  }
})

# Combine means into a single data frame
df_deg_mean2 <- do.call(rbind, means_list)
# Remove first row(is just OC)
df_deg_mean2 <- df_deg_mean2[-1,] %>%
  # Difference in stack voltage per phase of deg test [V]
  mutate(deg_V = Stack_V.mean_start - Stack_V.mean_end) %>%
  # Voltage degradation per 1000h [V/1000h]
  mutate(deg_V_1000h = 1000*deg_V / as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours"))) %>%
  # Percent voltage degradation per 1000h [V/1000h] (difference in voltage as percentage of Voltage at beginning of degradation test)
  mutate(deg_pV_1000h = label_percent()((deg_V/df_deg_mean2$Stack_V.mean_start[df_deg_mean2$Phase == "Before"])*(1000/as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours"))))) %>%
  # Compensated percent voltage degradation per 1000h [V/1000h] (difference in voltage as percentage of Voltage at beginning of degradation test)
  mutate(deg_pV_comp_1000h = label_percent()(((Stack_V_comp.mean_start - Stack_V_comp.mean_end) / df_deg_mean2$Stack_V_comp.mean_start[df_deg_mean2$Phase == "Before"] )*(1000/as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours"))))) %>%
  # Percentage decrease in equivalent resistance
  mutate(deg_pR_1000h = label_percent()(( (R.mean_start - R.mean_end)/ df_deg_mean2$R.mean_start[df_deg_mean2$Phase == "Before"] )*(1000/as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours"))))) %>%
  # Percentage point decrease in net system efficiency (Absolute)
  mutate(deg_eff_1000h = label_percent(scale = 1,)(( (Elec_eff_net.mean_start - Elec_eff_net.mean_end))*(1000/as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours"))))) %>%
  # Percentage decrease in electrical efficiency
  mutate(deg_peff_1000h = label_percent()(( (Elec_eff_net.mean_start - Elec_eff_net.mean_end)/ df_deg_mean2$Elec_eff_net.mean_start[df_deg_mean2$Phase == "Before"] )*(1000/as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours")))))

rm(r, run_starts, run_ends, means_list)

# Weighted mean of degradation per testphase (with and without motion and before and after thermal cycle)
df_deg_mean2_grouped <- df_deg_mean2[-c(6,16),] %>% # Remove rows with short timeframe (during lunchtime)
  group_by(Phase, Testphase) %>%
  # summarise(deg_V_1000h = mean(deg_V_1000h)) %>%
  summarise(deg_V_1000h.weighted = weighted.mean(deg_V_1000h, as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours"))),
            deg_pV_1000h.weighted = label_percent(accuracy=0.01)(weighted.mean(as.numeric(sub("%","",deg_pV_1000h))/100, as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours")))), # convert percentage value back to numeric before calculation
            deg_eff_1000h.weighted = label_percent(accuracy=0.01)(weighted.mean(as.numeric(sub("%","",deg_eff_1000h))/100, as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours")))),
            deg_peff_1000h.weighted = label_percent(accuracy=0.01)(weighted.mean(as.numeric(sub("%","",deg_peff_1000h))/100, as.numeric(difftime(Timestamp.mean_end, Timestamp.mean_start, units="hours")))),
  ) %>%
  arrange(desc(Phase))



# Stochastic estimation of degradation

# Define two dataframes to retrieve random data points from before and after first phase of degradation test
# 10h of steady operation before degradation experiment (without filling)
begin <- force_tz(as.POSIXct("2023-02-26  18:00:00"), tzone = "CET", roll = FALSE)
end <- force_tz(as.POSIXct("2023-02-27  04:00:00"), tzone = "CET", roll = FALSE)
df_first <- df_all %>%
  dplyr::filter(Timestamp > begin, Timestamp < end)
df_first_nofill <- df_first
filling <- which(df_first$SDpump_I > 200)
for (i in filling){                         
  # Find the range of times to remove
  start_time <- df_first[i, "Timestamp"] - 60
  end_time <- df_first[i, "Timestamp"] + 90
  # Remove the rows within the specified range
  df_first_nofill <- df_first_nofill %>% dplyr::filter(Timestamp < start_time | Timestamp > end_time)
}


# 10h of steady operation after first phase of degradation experiment (so before unplanned shutdown)
begin <- force_tz(as.POSIXct("2023-03-06  19:00:00"), tzone = "CET", roll = FALSE)
end <- force_tz(as.POSIXct("2023-03-07  05:00:00"), tzone = "CET", roll = FALSE)
df_last <- df_all %>%
  dplyr::filter(Timestamp > begin, Timestamp < end)
df_last_nofill <- df_last
filling <- which(df_last$SDpump_I > 200)
for (i in filling){                         
  # Find the range of times to remove
  start_time <- df_last[i, "Timestamp"] - 60
  end_time <- df_last[i, "Timestamp"] + 90
  # Remove the rows within the specified range
  df_last_nofill <- df_last_nofill %>% dplyr::filter(Timestamp < start_time | Timestamp > end_time)
}

# Create a function to calculate the slope between two points (in percentage per second)
calc_slope <- function(x1, y1, x2, y2) {
  (y2 - y1) / (x2 - x1)
}

# Define the number of random samples to take
num_samples <- 100000

# Create an empty vector to store the slopes
degradation <- numeric(num_samples)

# Loop over the number of samples and calculate a random slope
for (i in 1:num_samples) {
  
  # Choose two random points within the sub-window
  x1 <- sample(df_first_nofill$Elapsed_time, size = 1)
  x2 <- sample(df_last_nofill$Elapsed_time, size = 1)
  
  # Calculate the corresponding y values
  y1 <- df_first_nofill$Elec_eff_stack[df_first_nofill$Elapsed_time == x1]
  y2 <- df_last_nofill$Elec_eff_stack[df_last_nofill$Elapsed_time == x2]
  
  # Calculate the slope
  degradation[i] <- -3600 * 1000 * calc_slope(x1, y1, x2, y2) # in absolute efficiency per kh
  
}

# Confidence interval
z <- qnorm(0.95)
error <- z*sd(degradation)/sqrt(num_samples)

# Print the results of stochastic set
cat("Mean efficiency degradation (stochastic):", mean(degradation), "\n")
cat("Standard deviation:", sd(degradation), "\n")
cat("Min degradation:", min(degradation), "\n")
cat("Max degradation:", max(degradation), "\n")
cat("Confidence interval ",pnorm(z),": ", round(mean(degradation),3)," +- ", round(error,3), "%./kh \n")

# Mean from 10 hour before and 10 hour after testing

degradation <- (mean(df_first_nofill$Elec_eff_net) - mean(df_last_nofill$Elec_eff_net)) / (mean(df_last_nofill$Elapsed_time) - mean(df_first_nofill$Elapsed_time))*60*60*1000
cat("Mean efficiencydegradation (singular):", degradation, "%./kh \n")

rm(df_first,df_last,x1,x2,y1,y2,z,error)

# Gaussian processes to calculate confidence interval
df_deg_filtered %>% dplyr::filter(Phase == 'Before') %>% select(Elapsed_time,Elec_eff_net) %>% write.csv("df_deg_filtered.csv", row.names=FALSE)

# Plots of degradation experiment
# Remove data during shutdown
begin <- force_tz(as.POSIXct("2023-03-07  08:30:00"), tzone = "CET", roll = FALSE)
end <- force_tz(as.POSIXct("2023-03-09  09:04:43"), tzone = "CET", roll = FALSE)

# Voltage plot
df_deg_filtered %>%
  # subset(Timestamp < begin | Timestamp > end) %>%           # Remove data from shutdown
  # subset(Elapsed_time > 2000 & Elapsed_time < 958000) %>%     # Remove part where moving average looks weird
  dplyr::filter(Phase == 'Before') %>%                        # Only keep data before shut down
  subset(Elapsed_time > 2000 & Elapsed_time < 681000) %>%   # Remove part where moving average looks weird
  ggplot(aes(x= Elapsed_time/3600)) +
  ylim(55,57)+
  geom_line(aes(y=Stack_V, color = "Measured"))+
  geom_line(aes(y=Stack_V_avg, color = "Low-pass filtered"))+
  geom_line(aes(y=rollmean(Stack_V, 25000, na.pad = TRUE, align = "right"), color = "Moving average")) +
  labs(x = "Time [h]", y ="Voltage [V]")+
  scale_color_manual(name="Legend", breaks=c('Measured', 'Low-pass filtered', 'Moving average'), values=c("black",default_colors[1],default_colors[2]))
ggsave("2 Combined results/Degredation/Degration_nofilling_V_time.png",width=width_double_graph, height=height_graph)

# Compensated voltage plot
df_deg_filtered %>%
  # subset(Timestamp < begin | Timestamp > end) %>% # Remove shutdown data
  dplyr::filter(Phase == 'Before') %>%              # Only keep data before shut down
  ggplot(aes(x= Elapsed_time/3600)) +
  ylim(55,57)+
  geom_line(aes(y=Stack_V, color="Measured"), alpha = 0.4)+
  geom_line(aes(y=Stack_V_comp, color="Compensated for current"), alpha = 0.4)+
  labs(x = "Time [h]", y ="Voltage [V]")+
  scale_color_manual(name="Legend", values=c("red","black"),guide = guide_legend(reverse = TRUE))
ggsave("2 Combined results/Degredation/Voltage_nofilling_compensated.png",width=width_double_graph, height=height_graph)

# Current plot
df_deg_filtered %>%
  # subset(Timestamp < begin | Timestamp > end) %>%           # Remove data from shutdown
  # subset(Elapsed_time > 2000 & Elapsed_time < 955500) %>%   # Remove part where moving average looks weird
  subset(Elapsed_time > 2000 & Elapsed_time < 681000) %>%   # Remove part where moving average looks weird
  dplyr::filter(Phase == 'Before') %>%                      # Only keep data before shut down
  ggplot(aes(x= Elapsed_time/3600)) +
  # ylim(29.5,30.1)+
  geom_line(aes(y=Stack_I, color = "Measured"))+
  geom_line(aes(y=Stack_I_avg, color = "Low-pass filtered"))+
  geom_line(aes(y=rollmean(Stack_I, 25000, na.pad = TRUE, align = "right"), color = "Moving average")) +
  labs(x = "Time [h]", y ="Current [I]")+
  scale_color_manual(name="Legend", breaks=c('Measured', 'Low-pass filtered', 'Moving average'), values=c("black", default_colors[1], default_colors[2]))
ggsave("2 Combined results/Degredation/Degration_nofilling_I_time.png",width=width_double_graph, height=height_graph)

# Efficiency plot
df_deg_filtered %>%
  # subset(Timestamp < begin | Timestamp > end) %>%           # Remove data from shutdown
  # subset(Elapsed_time > 2000 & Elapsed_time < 955500) %>%   # Remove part where moving average looks weird
  subset(Elapsed_time > 2000 & Elapsed_time < 681000) %>%   # Remove part where moving average looks weird
  dplyr::filter(Phase == 'Before') %>%                      # Only keep data before shut down
  ggplot(aes(x= Elapsed_time/3600)) +
  # ylim(29.5,30.1)+
  geom_line(aes(y=Elec_eff_net, color = "Measured"))+
  geom_line(aes(y=Elec_eff_net_avg, color = "Low-pass filtered"))+
  geom_line(aes(y=rollmean(Elec_eff_net, 25000, na.pad = TRUE, align = "right"), color = "Moving average")) +
  labs(x = "Time [h]", y ="Net electric efficiency [%]")+
  scale_color_manual(name="Legend", breaks=c('Measured', 'Low-pass filtered','Moving average'), values=c("black",default_colors[1], default_colors[2]))
ggsave("2 Combined results/Degredation/Degration_nofilling_eff_time.png",width=width_double_graph, height=height_graph)

rm(plot_deg)
rm(df_first, df_first_nofill,df_last,df_last_nofill)