library(ggplot2)
library(gridExtra) # Plot several plots in one grid
library(here) # Package for current file location for saving files
library(dplyr) # is needed to use function %>%
library(reshape2) # for reshaping dataframe
library(cowplot) #put plots together with option to align
library(tidyr) # for function pivot_wider
library(gt) # Layout of tables
library(zoo) # For moving average
library(lubridate) # Timezones
library(signal) # For average of osciallations
library(scales) # for percentage (make sure you use dplyr::filter for filter function, otherwise it doesn't work)
library(viridis) # colorscale
library(paletteer) # colorscale
library(RColorBrewer) # colorscale
library(hexbin)       # density plot
library(ggridges)    # for ridgeline plot
library(stringr)

# --- Define formats ---
width_graph = 9
width_double_graph = 12
height_graph = 6
fontsize_figures = 14
linesize1 = 1
linesize2 = 1.5*0.5

# Set theme for the whole script
theme_set(theme_bw() + 
            theme(text = element_text(size = fontsize_figures)) # *1.5 for some specific graph
)

cbbPalette <- c("#000000", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")
colorscale <- paletteer_c("grDevices::YlGnBu", 30)
# Own colorpalette (not used)
# colorPalette <- c("#CC0000", "#4F81BD", "#9BBB59", "#FFC000", # "#808080") #Red, blue, green, yellow, grey
default_colors <- c("#619CFF", "#F8766D", "#00BA38", "#808080")
thesis_palette <- c("#495FA8","#DB343B","#41A48F","#EEC843","#36362C") # own palette

testphase_s.labs <- c('X' = "Inclination around X", 'Y' = "Inclination around Y")

testphase_d_grid.labs <- c('X' = "Rotation around X", 'Y' = "Rotation around Y", '15' = "15°", '22.5' = "22.5°", '30' = "30°")

loadcase.labs <- c('X' = "Rotation around X", 'Y' = "Rotation around Y", 'Full_load' = "Full load", 'Part_load' = "Part load")

# Dummy values (not the nominal values of used the SOFC system)
nominal_values <- data.frame(
  AC_Power = 1500, # [W]
  Air_flow = 100, # [SLM]
  Burner_T = 500, # [°C]
  Elec_eff_net = 50, # [%]
  Elec_eff_stack = 50, # [%]
  FU = 100, # [%]
  Gas_flow = 5, # [SLM]
  S2C = 1, # [-]
  Stack_I = 50, # [A]
  Stack_out_T = 500, # [°C]
  Stack_V = 50, # [V]
  Steam_flow = 5 # [SLM]
)

ggsave_cc = function(plot, file, dir, fig_width = 9, fig_height = 7, write_data = TRUE) {
  for (ext in c("png", "svg")) {
    ggsave(filename = sprintf("%s/%s.%s", dir, file, ext),
           plot = plot,
           width = fig_width, height = fig_height)
  }
  if(write_data) {
    write_csv(plot$data, file = sprintf("%s/%s_data.csv", dir, file))
  }
}

# --- Static testing ---
# Absolute plot
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  plot <- df_static_test_nofill %>%
  ggplot(aes(x = reorder(as.character(Inclination_set),-Inclination_set), y = !!col_sym, fill = Testphase)) +
    geom_boxplot(alpha = 0.6)+
    labs(x= "Inclination [deg]", y = parameter_labels[[col_sym]])+
    theme(legend.position = "none") +
    facet_wrap(~Direction, labeller = as_labeller(testphase_s.labs)) + # Remove free scales to see them in one graph
    scale_fill_manual(values = thesis_palette)
  ggsave(filename=here("2 Combined results/Static/Inclination/Absolute",paste("BG15v2_S_inc_box",col,".png",sep="_")),width=width_double_graph, height=height_graph)
} 
# Normalised plot
for(col in colnames(nominal_values)){
  col_sym <- sym(col) # changing col data type to symbol
  plot <- df_static_test_nofill %>% 
  ggplot(aes(x = reorder(as.character(Inclination_set),-Inclination_set), y = !!col_sym/nominal_values[[col_sym]], fill = Testphase)) +
    geom_boxplot(alpha = 0.6)+
    labs(x= "Inclination [deg]", y = parameter_labels[[col_sym]])+
    scale_y_continuous(labels = scales::percent_format(accuracy = 0.1))+
    theme(legend.position = "none") +
    facet_wrap(~Direction, labeller = as_labeller(testphase_s.labs)) + # Remove free scales to see them in one graph
    scale_fill_manual(values = thesis_palette)
  ggsave(filename=here("2 Combined results/Static/Inclination/Normalised",paste("BG15v2_S_inc_box",col,".png",sep="_")),width=width_double_graph, height=height_graph)  
}

# --- Dynamic testing ---

# System parameters over elapsed time for different testphases, inclination angle and RPM using for loop for all parameters
df_dynamic_test_time <- df_dynamic_test %>%                                           # Reset elapsed time for change in inclination
  mutate(Inc_diff = Inclination_set - lag(Inclination_set, default = dplyr::first(Inclination_set))) %>% # Add column that indicates change in inclination
  mutate(RPM_diff = RPM - lag(RPM, default = dplyr::first(RPM))) %>%                         # Add column that indicates change in rpm
  group_by(cumsum(RPM_diff != 0), Inclination_set) %>%                                    # Logic that identifies a change in inclination setting or new test phase
  mutate(T_reset = Elapsed_time - first(Elapsed_time))
# Full load over time
# Absolute plot
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    subset(!(Testphase == 'BG15v2_D_X_PL50' | Testphase == 'BG15v2_D_Y_PL50')) %>% # Exclude part-load for this plot
    dplyr::filter(Period<100) %>%
    arrange(Period) %>%
    ggplot() +
    xlim(0, 600)+
    geom_line(aes(x = T_reset, y = !!col_sym, colour = Period))+
    labs(x= "Time [s]", y = parameter_labels[[col_sym]], colour = "Period")+
    ggh4x::facet_grid2(Direction~Inclination_set, scales = "free_y",  space = "free_y", labeller = as_labeller(testphase_d_grid.labs)) +
    scale_colour_gradientn(colors = colorscale, name='Period [s]', label = function(x) sprintf("%.0f", x))
  ggsave(filename=here("2 Combined results/Dynamic/Elapsed time/Full load/Absolute",paste("BG15v2_D_time",col,".png",sep="_")),width=width_double_graph, height=height_graph)
}
# Normalised plot
for(col in colnames(nominal_values)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    subset(!(Testphase == 'BG15v2_D_X_PL50' | Testphase == 'BG15v2_D_Y_PL50')) %>% # Exclude part-load for this plot
    dplyr::filter(Period<100) %>%
    arrange(Period) %>%
    ggplot() +
    xlim(0, 600)+
    geom_line(aes(x = T_reset, y = !!col_sym/nominal_values[[col_sym]], colour = Period))+
    scale_y_continuous(labels = scales::percent_format(accuracy = 0.1))+
    labs(x= "Time [s]", y = parameter_labels[[col_sym]], colour = "Period")+
    ggh4x::facet_grid2(Direction~Inclination_set, scales = "free_y",  space = "free_y", labeller = as_labeller(testphase_d_grid.labs)) +
    scale_colour_gradientn(colors = colorscale, name='Period [s]', label = function(x) sprintf("%.0f", x))
  ggsave(filename=here("2 Combined results/Dynamic/Elapsed time/Full load/Normalised",paste("BG15v2_D_time",col,".png",sep="_")),width=width_double_graph, height=height_graph)
}

# Part-load and full load for 30 degrees over time
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    subset((Inclination_set == 30)) %>% # Exclude part-load for this plot 
    dplyr::filter(Period<100) %>%
    arrange(Period) %>%
    ggplot() +
    xlim(0, 600)+
    geom_line(aes(x = T_reset, y = !!col_sym, colour = Period))+
    labs(x= "Time [s]", y = parameter_labels[[col_sym]], colour = "Period")+
    ggh4x::facet_grid2(Loadcase~Direction, scales = "free_y",  space = "free_y", labeller = as_labeller(loadcase.labs)) +
    scale_colour_gradientn(colors = colorscale, name='Period [s]', label = function(x) sprintf("%.0f", x))
  ggsave(filename=here("2 Combined results/Dynamic/Elapsed time/Part load",paste("BG15v2_D_time",col,"part.png",sep="_")),width=width_double_graph, height=height_graph)
}

# --- Density plots ---
# Violin plot of full-load
# Absolute
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    dplyr::filter(Period<100) %>%
    ggplot(aes(x = factor(Inclination_set), y = !!col_sym)) +
    geom_violin(aes(fill = factor(Inclination_set)), alpha = 0.4, size = linesize2)+
    geom_boxplot(aes(fill = factor(Inclination_set)), width=0.15)+
    ggh4x::facet_grid2(Loadcase~Direction, scales = "free_y", space = "free_y", labeller = as_labeller((loadcase.labs))) +
    labs(x = "Inclination setting [°]", y =parameter_labels[[col_sym]]) +
    scale_fill_discrete(name = "Inclination setting", breaks = names(testphase_d_grid.labs[3:5]), labels = testphase_d_grid.labs[3:5])
  ggsave(filename=here("2 Combined results/Dynamic/Density/Violin all/Absolute",paste("BG15v2_D_violin_all",col,".png",sep="_")),width=width_graph, height=height_graph)
}
# Normalised
for(col in colnames(nominal_values)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    dplyr::filter(Period<100) %>%
    ggplot(aes(x = factor(Inclination_set), y = !!col_sym/nominal_values[[col_sym]])) +
    scale_y_continuous(labels = scales::percent_format(accuracy = 0.1))+
    geom_violin(aes(fill = factor(Inclination_set)), alpha = 0.4, size = linesize2)+
    geom_boxplot(aes(fill = factor(Inclination_set)), width=0.15)+
    ggh4x::facet_grid2(Loadcase~Direction, scales = "free_y", space = "free_y", labeller = as_labeller((loadcase.labs))) +
    labs(x = "Inclination setting [°]", y =parameter_labels[[col_sym]]) +
    scale_fill_manual(values = c(thesis_palette[2],thesis_palette[3],thesis_palette[1]), name = "Inclination setting", breaks = names(testphase_d_grid.labs[3:5]), labels = testphase_d_grid.labs[3:5])
  ggsave(filename=here("2 Combined results/Dynamic/Density/Violin all/Normalised",paste("BG15v2_D_violin_all",col,".png",sep="_")),width=width_graph, height=height_graph)
}

# Violin plot for period groups and inclination setting
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    dplyr::filter(Period<100) %>%
    dplyr::filter(Loadcase == 'Full_load') %>%
    mutate(Period.group = cut(Period, breaks = seq(0, 80, 10))) %>%
    ggplot(aes(x = Period.group, y = !!col_sym)) +
    geom_violin(aes(fill = Direction), alpha = 0.6, size = linesize2) +
    scale_fill_manual(values = thesis_palette) +
    ggh4x::facet_grid2(Inclination_set~Testphase, scales = "free_y", space = "free_y", labeller = as_labeller((testphase_d_grid.labs))) +
    labs(x = "Period interval [s]", y =parameter_labels[[col_sym]])
  ggsave(filename=here("2 Combined results/Dynamic/Density/Violin inc",paste("BG15v2_D_violin_all",col,".png",sep="_")),width=width_graph, height=height_graph)
}

# Violin plot for period groups and part load
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    dplyr::filter(Period<100) %>%
    mutate(Period.group = cut(Period, breaks = seq(0, 80, 10))) %>%
    ggplot(aes(x = Period.group, y = !!col_sym)) +
    geom_violin(aes(fill = Direction), alpha = 0.6, size = linesize2)+
    ggh4x::facet_grid2(Loadcase~Direction, scales = "free_y", space = "free_y", labeller = as_labeller((loadcase.labs)))+
    labs(x = "Period interval [s]", y =parameter_labels[[col_sym]])
  ggsave(filename=here("2 Combined results/Dynamic/Density/Violin part",paste("BG15v2_D_violin_all",col,".png",sep="_")),width=width_graph, height=height_graph)
}
# Ridged diagram of measurements for different intervals of period
for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  df_dynamic_test_time %>%
    dplyr::filter(Period<100) %>%
    dplyr::filter(Loadcase == 'Full_load') %>%
    mutate(Period.group = cut(Period, breaks = seq(0, 80, 10))) %>%
    ggplot(aes(x = !!col_sym, y = Period.group, fill=Direction)) +
    geom_density_ridges(scale = 1.6, bandwidth = 0.05, rel_min_height = 0.000001, alpha = 0.6) +
    labs(x = parameter_labels[[col_sym]], y = "Period interval [s]", title = 'Density of experimental measurements')+
    theme(panel.border = element_blank())
  ggsave(filename=here("2 Combined results/Dynamic/Density/Ridgeline",paste("BG15v2_D_ridge",col,".png",sep="_")),width=width_graph, height=height_graph)
}

# --- Calculate and plot maximum deviations in measure data ---
period <- unique(na.omit(df_tests$Period))
period_rows <- round(0.5*period)

df_maxima = data.frame()


# Find maximum deviations in normal operation
# Define 40 minutes of normal operation (without toggle of airflow and safety measurements for sensor)
begin <- force_tz(as.POSIXct("2023-02-10 19:15:00"), tzone = "CET", roll = FALSE)
end   <- force_tz(as.POSIXct("2023-02-10 19:55:00"), tzone = "CET", roll = FALSE)

df_normal <- df %>%
  dplyr::filter(Timestamp >= begin, Timestamp <= end)

for (col in colnames(parameter_labels)){
  col_sym <- sym(col)
  df_normal_max <- df_normal %>%
    mutate(col_diff = abs(!!col_sym - lead(!!col_sym, n = min(period_rows))))
  normal_max <- round(max(df_normal_max$col_diff, na.rm = TRUE),3)
  col_max <- c(parameter_labels[[col_sym]], "Normal", normal_max)
  df_maxima <- rbind(df_maxima, col_max)
}

colnames(df_maxima) <- c('Variable','State','Value')

# Find maximum deviations in static test
# Exclude water tank filling
df_static_test_nofill <- df_static_test
filling <- which(df_static_test$SDpump_I > 200)
for (i in filling){
  # Find the range of times to remove
  start_time <- df_static_test[i, "Timestamp"] - 60
  end_time <- df_static_test[i, "Timestamp"] + 90
  # Remove the rows within the specified range
  df_static_test_nofill <- df_static_test_nofill %>% dplyr::filter(Timestamp < start_time | Timestamp > end_time)
}

for (col in colnames(parameter_labels)){
  df_static_test_max = data.frame()
  col_sym <- sym(col) # changing col data type to symbol
  for (p in unique(df_static_test$Testphase)){
    for (a in unique(df_static_test$Inclination_set)){
      df_static_test_max_t <- df_static_test_nofill %>%
        dplyr::filter(Testphase == p, Inclination_set == a) %>%
        mutate(col_diff = abs(!!col_sym - lead(!!col_sym, n = min(period_rows)))) %>%
        mutate(col_diff_p = abs(!!col_sym - lead(!!col_sym, n = min(period_rows))) / mean(!!col_sym)) %>%
        group_by(Testphase,Inclination_set) %>%
        summarise(col_max = max(col_diff, na.rm = TRUE), col_max_p = label_percent(0.01)(max(col_diff_p, na.rm = TRUE)))
      df_static_test_max <- rbind(df_static_test_max, df_static_test_max_t)
    }
  }
  col_max <- c(parameter_labels[[col_sym]],"Static",round(max(df_static_test_max$col_max),3))
  df_maxima <- rbind(df_maxima,col_max)
}


# Find maximum deviations in dynamic test
# Exclude water tank filling
df_dynamic_test_nofill <- df_dynamic_test
filling <- which(df_dynamic_test$SDpump_I > 200)
for (i in filling){
  # Find the range of times to remove
  start_time <- df_dynamic_test[i, "Timestamp"] - 60
  end_time <- df_dynamic_test[i, "Timestamp"] + 90
  # Remove the rows within the specified range
  df_dynamic_test_nofill <- df_dynamic_test_nofill %>% dplyr::filter(Timestamp < start_time | Timestamp > end_time)
}

for (col in colnames(parameter_labels)){
  df_dynamic_test_max = data.frame()
  col_sym <- sym(col) # changing col data type to symbol
  for (p in unique(df_dynamic_test$Testphase)){
    for (a in unique(df_dynamic_test$Inclination_set)){
      for (s in seq_along(unique(df_dynamic_test$Period))){
        df_dynamic_test_max_t <- df_dynamic_test_nofill %>%
          dplyr::filter(Testphase == p, Inclination_set == a, Period == period[s]) %>%
          mutate(col_diff = abs(!!col_sym - lead(!!col_sym, n = period_rows[s]))) %>%
          mutate(col_diff_p = abs(!!col_sym - lead(!!col_sym, n = period_rows[s])) / mean(!!col_sym)) %>%
          group_by(Testphase,Period,Inclination_set) %>%
          summarise(Direction = first(Direction), col_max = max(col_diff, na.rm = TRUE), col_max_p = label_percent(0.01)(max(col_diff_p, na.rm = TRUE)))
        df_dynamic_test_max <- rbind(df_dynamic_test_max, df_dynamic_test_max_t)
        df_dynamic_test_max_wide <- pivot_wider(df_dynamic_test_max, names_from = Testphase, values_from = c(col_max, col_max_p),
                                                names_glue = "{.value}.{sub('.*D_', '', Testphase)}")
        
      }
    }
  }
  # Append dataframe with maxima of all variables
  col_max <- c(parameter_labels[[col_sym]],"Dynamic",round(max(df_dynamic_test_max$col_max),3))
  df_maxima <- rbind(df_maxima,col_max)
# }  #Use when you want to skip plotting

  # Create line plot for RPM, inclination and rotation axis
  df_dynamic_test_max %>%
    subset(Testphase == 'BG15v2_D_X' | Testphase == 'BG15v2_D_Y') %>% # Remove partload test
    dplyr::filter(Period<100) %>%
    arrange(desc(Inclination_set)) %>%
      ggplot(aes(y=col_max, x=Period)) + 
      # geom_line() +
      geom_point(aes(colour = factor(Inclination_set))) +
      geom_smooth(aes(colour = factor(Inclination_set), fill = factor(Inclination_set)), level = 0.75, show.legend = FALSE) + # Default alpha is 0.4
      labs(y = paste("Maximum deviation in",parameter_labels[[col_sym]]), fill = "Inclination") +
      scale_x_continuous(name="Oscillation period [s]", trans='log10') +
      facet_wrap(~Direction, labeller = as_labeller(testphase_d_grid.labs[1:2]))+
      scale_colour_manual(values = c(thesis_palette[1],thesis_palette[3],thesis_palette[2]), name = "Inclination setting", breaks = rev(names(testphase_d_grid.labs[3:5])), labels = rev(testphase_d_grid.labs[3:5]))
    ggsave(filename=here("2 Combined results/Dynamic/Deviation/Full load",paste("BG15_H_D_max",col,".png",sep="_")),width=width_double_graph, height=height_graph)
  
  # Create line plot for RPM and rotation axis for part load and full-load at 30 degrees only
  df_dynamic_test_max %>%
    mutate(Loadcase = if_else(endsWith(Testphase,'PL50'), 'Part_load', 'Full_load')) %>%
    mutate(Direction = if_else(grepl('Y',Testphase,fixed=TRUE),'Y','X') ) %>%
    dplyr::filter(Period<100) %>%
    dplyr::filter(Inclination_set == 30) %>%
      # order_by(Inclination,RPM)
      ggplot(aes(y=col_max, x=Period, linetype = Loadcase)) + 
      geom_point(aes(shape = Loadcase)) +
      geom_smooth(level = 0.75, color = default_colors[1], fill = default_colors[1]) + 
      labs(y = paste("Maximum deviation in",parameter_labels[[col_sym]]), fill = "Inclination") +
      scale_x_continuous(name="Oscillation period [s]", trans='log10') +
      facet_wrap(~Direction, labeller = as_labeller(testphase_d_grid.labs[1:2]))+
      scale_shape_discrete(name = "Measured", breaks = c('Full_load','Part_load'), labels = loadcase.labs[3:4])+
      scale_linetype_discrete(name = "Fitted", breaks = c('Full_load','Part_load'), labels = loadcase.labs[3:4])
    ggsave(filename=here("2 Combined results/Dynamic/Deviation/Part load",paste("BG15_H_D_max_part",col,".png",sep="_")),width=width_double_graph, height=height_graph)
  
  # Create line plot for RPM and rotation axis for part load and full-load (all data)
    df_dynamic_test_max %>%
      mutate(Loadcase = if_else(endsWith(Testphase,'PL50'), 'Part_load', 'Full_load')) %>%
      mutate(Direction = if_else(grepl('Y',Testphase,fixed=TRUE),'Y','X') ) %>%
      dplyr::filter(Period<100) %>%
      # order_by(Inclination,Period) %>%
      ggplot(aes(y=col_max, x=Period, linetype = Loadcase, colour = factor(Inclination_set))) + 
      geom_line() +
      geom_point() +
      labs(y = paste("Maximum deviation in",parameter_labels[[col_sym]]), fill = "Inclination") +
      scale_x_continuous(name="Oscillation period [s]", trans='log10') +
      facet_wrap(~Direction, labeller = as_labeller(testphase_d_grid.labs[1:2]))+
      scale_colour_discrete(name = "Inclination setting", breaks = rev(names(testphase_d_grid.labs[3:5])), labels = rev(testphase_d_grid.labs[3:5]))+
      scale_linetype_discrete(name = "Loadcase", breaks = c('Full_load','Part_load'), labels = loadcase.labs[3:4])
    ggsave(filename=here("2 Combined results/Dynamic/Deviation/Both",paste("BG15_H_D_max_part",col,".png",sep="_")),width=width_double_graph, height=height_graph)
    
      
  df_dynamic_test_max_wide <- df_dynamic_test_max_wide[,c(1,2,3,5,4,6)] %>%
    mutate_at(vars(col_max.X, col_max.Y), funs(round(., 3)))
  
}


# Plot selection of variables as percentage of maximum per operational state
plot_variables <- c('AC_Power','Burner_T','Elec_eff_stack','Elec_eff_net','FU','Gas_flow','S2C','Stack_I','Stack_out_T','Stack_Power','Stack_V','Steam_flow')
col.name <- parameter_labels[, match(plot_variables, names(parameter_labels))]

df_maxima$Value <- as.numeric(df_maxima$Value)
df_maxima$State <- factor(df_maxima$State, levels = c('Normal', 'Static', 'Dynamic')) # Define right order for graph
df_maxima %>%
  subset (Variable %in% col.name) %>%
  group_by(Variable) %>%
  dplyr::filter(Variable != 'Burner temp [°C]') %>%
  mutate(Value_p = Value/min(Value)) %>%  # max(Value) to normalise for dynamic condition
  mutate(Variable_no_unit = str_remove(Variable, "\\[.*?\\]")) %>%  # remove unit with regex
  mutate(unit = gsub("\\[|\\]", "", regmatches(Variable, gregexpr("\\[.*?\\]", Variable))[[1]])) %>%  # extract unit with regex
  mutate(unit = str_remove(unit, "-")) %>%  # remove - with regex
  ggplot(aes(fill=State, y=Value_p, x=Variable_no_unit)) + 
  geom_bar(position="dodge", stat="identity")+
  geom_hline(aes(yintercept=1)) +
  geom_text(aes(0,1,label = "100%", vjust = -1, hjust=-0.14))+
  geom_text(aes(label=paste(round(Value/2,2),unit), hjust = -0.25), position = position_dodge(0.9), angle = 90)+
  theme(axis.text.x=element_text(angle=45,hjust=1,vjust=1))+
  scale_fill_manual(values=c(thesis_palette[3],thesis_palette[1],thesis_palette[2])) +
  labs(x = "Measured parameter", y ="Normalised maximum amplitude of fluctuations")+
  scale_y_continuous(labels = scales::percent,breaks = c(0,2,4,6,8,10), limits = c(0,10))
ggsave(filename=here("2 Combined results/All fases/Overview_of_maximum_deviations.png"),width=width_double_graph, height=height_graph)

df_maxima_wide <- pivot_wider(df_maxima, names_from = State, values_from = Value) %>%
  arrange(Variable)

gt(df_maxima_wide) %>%
  cols_width(Normal ~ px(100), Static ~ px(100), Dynamic ~ px(100)) %>%
  tab_header(title = "Maximum deviations in measured parameters") %>%
  gtsave(filename = here("2 Combined results/All fases/Table of maximum deviations.png"))


# ----- Plot acceleration test -----
df_platform <- data.frame(RPM = seq(400, 950, 50), a_abs_max = c(0.16, 0.2, 0.24, 0.29, 0.35, 0.41, 0.48, 0.55, 0.62, 0.7, 0.79, 0.88))

df_highRPM <- merge(df_other_test, df_platform[, c("RPM", "a_abs_max")], by="RPM")
start_time <- df_highRPM[1, 1]
df_highRPM$Elapsed_time <- df_highRPM$Timestamp - start_time
df_highRPM$Elapsed_time <- as.numeric(difftime(df_highRPM$Timestamp, df_highRPM$Timestamp[1], units = "secs"))

ylim.prim <- c(0, 1)  
ylim.sec <- c(0, 10)    
b <- diff(ylim.prim)/diff(ylim.sec)
a <- ylim.prim[1] - b*ylim.sec[1]

for(col in colnames(parameter_labels)){
  col_sym <- sym(col) # changing col data type to symbol
  # Plot over time
  plot1 <- ggplot(df_highRPM, aes(x = Elapsed_time, y = !!col_sym)) + 
    geom_line() +
    labs(title = "Measured parameter", x = 'Time [s]', y = parameter_labels[[col_sym]])
  
  plot2 <- ggplot(df_highRPM, aes(x = Elapsed_time)) + 
    geom_line(aes(y = a_abs_max), color = default_colors[1]) +
    geom_line(aes(y = a + Period*b), color = default_colors[2]) +
    scale_y_continuous(sec.axis = sec_axis(~ (. - a)/b, name = "Period [s]")) +
    labs(title = "Test condition", x = 'Time [s]', y = expression("Max acceleration of COG [" ~m/s^2 ~"]"))+
    theme(axis.text.y.left = element_text(color = default_colors[1]), 
          axis.title.y.left = element_text(color = default_colors[1]),         
          axis.text.y.right = element_text(color = default_colors[2]), 
          axis.title.y.right = element_text(color = default_colors[2])
    )
  plot_grid(plot1, plot2, ncol = 1, align = "v", axis = "tb")
  ggsave(filename=here("2 Combined results/Dynamic/High RPM",paste("BG15v2_D_highRPM_time",col,".png",sep="_")),width=width_graph, height=height_graph)
  
  #Absolute  
  df_highRPM %>%
    dplyr::filter(AC_Power>1470, AC_Power<1490) %>%     # remove water filling
    ggplot() +
    geom_boxplot(aes(x = a_abs_max, y = !!col_sym, fill = Direction, group = a_abs_max, alpha = 0.6))+
    scale_x_continuous(sec.axis = dup_axis(trans = function(x) {3.8254 * x^(-0.503)}, name = "Period [s]"))+
    labs(x= "Max acceleration of COG [m/s^2]", y = parameter_labels[[col_sym]])+
    scale_fill_manual(values = default_colors[1])+
    theme(legend.position="none")
  ggsave(filename=here("2 Combined results/Dynamic/High RPM/Absolute",paste("BG15v2_D_highRPM",col,".png",sep="_")),width=width_graph, height=height_graph)
} 
#Normalised
for(col in colnames(nominal_values)){
  col_sym <- sym(col) # changing col data type to symbol  
  df_highRPM %>%
    dplyr::filter(AC_Power>1470, AC_Power<1490) %>%     # remove water filling
    ggplot() +
    geom_boxplot(aes(x = a_abs_max, y = !!col_sym/nominal_values[[col_sym]], fill = Direction, group = a_abs_max, alpha = 0.6))+
    scale_y_continuous(labels = scales::percent_format(accuracy = 0.1))+
    scale_x_continuous(sec.axis = dup_axis(trans = function(x) {3.8254 * x^(-0.503)}, name = "Period [s]"))+
    labs(x= "Max acceleration of COG [m/s^2]", y = parameter_labels[[col_sym]])+
    scale_fill_manual(values = thesis_palette)+
    theme(legend.position="none")
  ggsave(filename=here("2 Combined results/Dynamic/High RPM/Normalised",paste("BG15v2_D_highRPM",col,".png",sep="_")),width=0.8*width_graph, height=0.8*height_graph)
}

# --- Other calculations ---
#Number of total hours in deg experiment
df_deg %>% dplyr::filter(Phase =='Before') %>% summarise(max = max(Elapsed_time))/60/60

#Numer of oscillation hours in deg experiment
nrow(df_deg[df_deg$Testphase == 'BG15v2_deg' & df_deg$Phase == 'Before', ])/60/60