library(ggplot2)
library(gridExtra) # Plot several plots in one grid
library(here) # Package for current file location for saving files
library(dplyr) # is needed to use function %>%
library(tidyr) # is needed to use function %>%
library(reshape2) # for reshaping dataframe
library(cowplot) #put plots together with option to align
library(lubridate) # Timezones
library(gganimate)   # animation
library(gifski)     # render gif
library(transformr) # Interpolate data in gif
library(magick)     # append gifs

# --- Plot specific tests or plots ---
# Plot of start of oscillating behavior
begin <- force_tz(as.POSIXct("2023-02-23 13:53:00"), tzone = "CET", roll = FALSE)
end   <- force_tz(as.POSIXct("2023-02-23 13:58:00"), tzone = "CET", roll = FALSE)
vline <- force_tz(as.POSIXct("2023-02-23 13:54:37"), tzone = "CET", roll = FALSE)

# Absolute
df %>%
  dplyr::filter(Timestamp >= begin, Timestamp <= end) %>%
  ggplot(aes(x = Timestamp, y = Stack_V)) +
  geom_line() +
  geom_vline(xintercept = vline, color = default_colors[1]) +
  labs(x = 'Timestamp [h]', y = 'Stack Voltage [V]') +
  scale_x_datetime(labels = date_format("%H:%M%n%d-%m"))
ggsave("2 Combined results/Specific/Start_oscillation.png",width=0.8*width_graph, height=0.8*height_graph)
# Normalised
df %>%
  dplyr::filter(Timestamp >= begin, Timestamp <= end) %>%
  ggplot(aes(x = Timestamp, y = Stack_V/nominal_values$Stack_V)) +
  geom_line() +
  geom_vline(xintercept = vline, color = default_colors[1]) +
  scale_y_continuous(labels = scales::percent_format(accuracy = 0.1))+
  labs(x = 'Timestamp [h]', y = 'Stack Voltage [V]') +
  scale_x_datetime(labels = date_format("%H:%M%n%d-%m"))
ggsave("2 Combined results/Specific/Normalised/Start_oscillation.png",width=0.8*width_graph, height=0.8*height_graph)


# Power drop during fuel utilization fluctuations
df_powerdrop <- df_dynamic_test_time %>%
  subset(Testphase == 'BG15v2_D_X' & Inclination_set == 30 & RPM == 150)

plot1 <- ggplot(df_powerdrop, aes(x = T_reset, y = Inclination)) + 
  geom_line() +
  labs(title = "Inclination", x = 'Time [s]', y = 'Inclination [°]')

plot2 <- ggplot(df_powerdrop, aes(x = T_reset)) + 
  geom_line(aes(y = Gas_flow)) +
  geom_line(aes(y = Gas_flow_req), linetype="dashed", color = default_colors[1]) +
  labs(title = "Gas flow", x = 'Time [s]', y = 'Gas flow [SLM]')

plot3 <- ggplot(df_powerdrop, aes(x = T_reset, y = Stack_V)) + 
  geom_line() +
  labs(title = "Voltage", x = 'Time [s]', y = 'Voltage [V]')

plot4 <- ggplot(df_powerdrop, aes(x = T_reset, y = AC_Power)) + 
  geom_line() +
  labs(title = "AC Power", x = 'Time [s]', y = 'Power [W]')

plot4_n <- ggplot(df_powerdrop, aes(x = T_reset, y = AC_Power/nominal_values$AC_Power)) + 
  geom_line() +
  scale_y_continuous(labels = scales::percent_format(accuracy = 1))+
  labs(title = "Normalised AC Power", x = 'Time [s]', y = 'Power [W]')

plot5 <- ggplot(df_powerdrop, aes(x = T_reset, y = Stack_I)) + 
  geom_line() +
  labs(title = "Stack current", x = 'Time [s]', y = 'Stack current [A]')

plot5_n <- ggplot(df_powerdrop, aes(x = T_reset, y = Stack_I/nominal_values$Stack_I)) + 
  geom_line() +
  scale_y_continuous(labels = scales::percent_format(accuracy = 1))+
  labs(title = "Normalised stack current", x = 'Time [s]', y = 'Stack current [A]')

plot6 <- ggplot(df_powerdrop, aes(x = T_reset, y = FU)) + 
  geom_line() +
  geom_hline(yintercept=85, linetype="dashed", color = default_colors[2])+
  labs(title = "Fuel utilisation", x = 'Time [s]', y = 'FU [%]')

plot6_n <- ggplot(df_powerdrop, aes(x = T_reset, y = FU/nominal_values$FU)) + 
  geom_line() +
  geom_hline(yintercept=85/nominal_values$FU, linetype="dashed", color = default_colors[2])+
  scale_y_continuous(labels = scales::percent_format(accuracy = 1))+
  labs(title = "Fuel utilisation", x = 'Time [s]', y = 'FU [%]')

plot7 <- ggplot(df_powerdrop, aes(x = T_reset, y = Air_flow)) + 
  geom_line() +
  labs(title = "Air flow", x = 'Time [s]', y = 'Air flow [SLM]')

plot8 <- ggplot(df_powerdrop, aes(x = T_reset, y = Elec_eff_net)) + 
  geom_line() +
  labs(title = "Efficiency", x = 'Time [s]', y = 'Net Electric efficiency [%]')

plot9 <- ggplot(df_powerdrop, aes(x = T_reset, y = S2C)) + 
  geom_line() +
  labs(title = "Steam to carbon", x = 'Time [s]', y = 'Steam to carbon ratio [-]')


# Combine the plots with a shared time axis (visualise power reduction)
# Absolute
plot_grid(plot1, plot6, plot5, plot4, ncol = 1, align = "v", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/Power_decrease.png"),width=width_double_graph, height=2*height_graph)
# Normalised
plot_grid(plot1, plot6, plot5_n, plot4_n, ncol = 1, align = "v", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/Normalised/Power_decrease.png"),width=width_graph, height=2*height_graph)


# Combine the plots with a shared time axis (order of control tree)
plot_grid(plot1, plot2, plot3, plot4, plot5, plot6, plot7, plot8, plot9, ncol = 1, align = "v", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/Power_decrease_control.png"),width=width_double_graph, height=4*height_graph)

# Signal of water steam dosing tank @@@ TO BE FINISHED
df_SD <- df_other_test %>%
  subset(Testphase == 'BG15v2_SD')

plot1 <- ggplot(df_SD, aes(x = Timestamp)) + 
  geom_line(aes(y = SDtank_LLH)) +
  geom_line(aes(y = SDpump_I)) +
  labs(title = "Steam dosing tank", x = 'Time [s]', y = 'Level sensor [-]') +
  scale_y_continuous(name = "First Axis", sec.axis = sec_axis(trans = ~.*10, name="Second Axis"))

# Oscillation behavior in worst case scenario
df_worstcase <- df_dynamic_test_time %>%
  subset(Testphase == 'BG15v2_D_X' & Inclination_set == 30 & RPM == 150)

plot1 <- ggplot(df_powerdrop, aes(x = T_reset, y = Inclination)) + 
  geom_line() +
  labs(title = "Inclination", x = 'Time [s]', y = 'Inclination [°]')

plot2 <- ggplot(df_powerdrop, aes(x = T_reset, y = FU)) + 
  geom_line() +
  geom_hline(yintercept=85, linetype="dashed", color = default_colors[2])+
  labs(title = "Fuel utilisation", x = 'Time [s]', y = 'FU [%]')

plot3 <- ggplot(df_powerdrop, aes(x = T_reset, y = Stack_I)) + 
  geom_line() +
  labs(title = "Stack current", x = 'Time [s]', y = 'Current [A]')

plot4 <- ggplot(df_powerdrop, aes(x = T_reset, y = AC_Power)) + 
  geom_line() +
  labs(title = "AC Power", x = 'Time [s]', y = 'Power [W]')


# Plot that shows fasechange with normalised parameters
plot_variables <- c('Inclination','Stack_V','Stack_I','Gas_flow','Elec_eff_net','FU')

begin <- force_tz(as.POSIXct("2023-02-23 10:25:00"), tzone = "CET", roll = FALSE)
end   <- force_tz(as.POSIXct("2023-02-23 10:26:00"), tzone = "CET", roll = FALSE)

df_dynamic_test %>%
  dplyr::filter(Timestamp >= begin, Timestamp <= end) %>%
  mutate_if(names(.) %in% plot_variables, funs(c(scale(.)))) %>% # normalise parameters
  gather(Parameter, Value, plot_variables, factor_key = TRUE) %>% # make longtable
  ggplot(aes(x = Timestamp)) +
  geom_line(aes(y = Value, colour = Parameter),size=linesize1)+
  scale_colour_manual(values=cbbPalette)+
  scale_linetype_manual(values=c("longdash","solid","solid","solid","solid","solid"))+
  labs(y= "Normalised parameter")

ggsave(filename=here("2 Combined results/Specific/Phasedifference.png"),width=width_graph, height=height_graph)

# Plot that shows daily check of fuel sensor
begin <- force_tz(as.POSIXct("2023-02-28 00:25:00"), tzone = "CET", roll = FALSE)
end   <- force_tz(as.POSIXct("2023-02-28 00:35:00"), tzone = "CET", roll = FALSE)

plot1 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp)) + 
  geom_line(aes(y = Gas_flow)) +
  geom_line(aes(y = Gas_flow_req), linetype="dashed", color = default_colors[2])+
  labs(title = "Gas flow", x = 'Time [h]', y = 'Gas flow [SLM]')

plot2 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = FU)) + 
  geom_line() +
  labs(title = "Fuel utilisation", x = 'Time [h]', y = 'FU [%]')

plot3 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = AC_Power)) + 
  geom_line() +
  labs(title = "AC Power", x = 'Time [h]', y = 'Power [W]')

plot_grid(plot1, plot2, plot3, ncol = 1, align = "h", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/Fuel_sensor.png"),width=width_double_graph, height=2*height_graph)

# Plot that shows hourly filling of steam dosing tank
begin <- force_tz(as.POSIXct("2023-02-27 21:00:00"), tzone = "CET", roll = FALSE)
end   <- force_tz(as.POSIXct("2023-02-27 23:00:00"), tzone = "CET", roll = FALSE)


plot1 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = SDpump_I/1000)) + 
  geom_line() +
  labs(title = "Current steam dosing tank pump", x = 'Time [h]', y = 'Current [I]')


plot2 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Stack_I)) + 
  geom_line() +
  labs(title = "Stack current", x = 'Time [h]', y = 'Current [I]')

plot3 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Stack_V)) + 
  geom_line() +
  labs(title = "Stack voltage", x = 'Time [h]', y = 'Voltage [V]')

plot4 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = AC_Power)) + 
  geom_line() +
  labs(title = "AC power", x = 'Time [h]', y = 'Power [W]')

plot_grid(plot1, plot2, plot3, plot4, ncol = 1, align = "h", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/SD_fill.png"),width=width_double_graph, height=2*height_graph)


# Plots that shows hourly toggle and daily check of air flow
begin <- force_tz(as.POSIXct("2023-03-06 04:05:00"), tzone = "CET", roll = FALSE)
end   <- force_tz(as.POSIXct("2023-03-06 09:05:00"), tzone = "CET", roll = FALSE)

plot1 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Air_flow)) + 
  geom_line() +
  labs(title = "Air flow", x = 'Time [h]', y = 'Air flow [SLM]')

plot2 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Elec_eff_net)) + 
  geom_line() +
  labs(title = "Efficiency", x = 'Time [h]', y = 'Net Electric efficiency [%]')

plot3 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Burner_T)) + 
  geom_line() +
  labs(title = "Burner temperature", x = 'Time [h]', y = 'Burner temperature [°C]')

plot_grid(plot1, plot2, plot3, ncol = 1, align = "h", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/Air_sensor.png"),width=width_double_graph, height=2*height_graph)

plot1 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Air_flow)) + 
  geom_line() +
  labs(title = "Air inflow", x = 'Time [h]', y = 'Air flow [SLM]')

plot2 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Gas_flow)) + 
  geom_line() +
  labs(title = "Fuel inflow", x = 'Time [h]', y = 'Gas flow [SLM]')

plot3 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Stack_V)) + 
  geom_line() +
  labs(title = "Stack Voltage", x = 'Time [h]', y = 'Voltage [V]')

plot4 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Stack_I)) + 
  geom_line() +
  labs(title = "Stack current", x = 'Time [h]', y = 'Current [A]')

plot5 <- ggplot(df_deg %>% dplyr::filter(Timestamp >= begin, Timestamp <= end), aes(x = Timestamp, y = Elec_eff_net)) + 
  geom_line() +
  labs(title = "Net electric efficiency", x = 'Time [h]', y = 'Efficiency [%]')

plot_grid(plot1, plot2, plot3, plot4, plot5, ncol = 1, align = "h", axis = "tb")
ggsave(filename=here("2 Combined results/Specific/All_checks.png"),width=width_double_graph, height=3*height_graph)

rm(plot1,plot2,plot3,plot4,plot5,plot6,plot7,plot8,plot9)

# Power drop during longer 150 RPM experiment
df_other_test %>%
  dplyr::filter(Testphase == 'BG15v2_150RPM') %>%
  ggplot(aes(x = Timestamp, y = AC_Power)) +
  geom_line() +
  labs(x = 'Timestamp [h]', y = 'AC Power [W]')
ggsave(filename=here("2 Combined results/Specific/Extended150RPMtest.png"),width=width_graph, height=height_graph)

# Power drop during all of dynamic experiment with X-axis and 30 degrees
begin_time <- force_tz(as.POSIXct("2023-02-23 14:45:00"), tzone = "CET", roll = FALSE)

inclination_periods <- df_dynamic_test %>% # create new dataframe that holds the test phases for different inclinations
  subset(Inclination_set == 30 & Direction == 'X' & Loadcase == 'Full_load') %>%
  dplyr::filter(Timestamp>begin_time) %>%
  group_by(Period) %>%
  dplyr::filter(Period>15, Period<26) %>%
  summarise(min_date = min(Timestamp), max_date= max(Timestamp), max_col = max(AC_Power))


df_dynamic_test %>%
  subset(Inclination_set == 30 & Direction == 'X' & Loadcase == 'Full_load') %>%
  dplyr::filter(Timestamp>begin_time) %>%
  ggplot(aes(x = Timestamp, y = AC_Power)) +
  geom_line() +
  labs(x = 'Timestamp [h]', y = 'AC Power [W]') +
  scale_x_datetime(labels = date_format("%H:%M%n%d-%m"))+
  geom_rect(data = inclination_periods, aes(NULL,NULL,xmin = min_date, xmax = max_date, ymin = -Inf, ymax = Inf), fill ='gray80', alpha= 0.3) + 
  geom_text(data = inclination_periods, aes(x = min_date+(max_date-min_date)/2, y = 1500, label = paste(sprintf("%.0f", Period)," s", sep = "")))+
  geom_text(data = inclination_periods, aes(x = begin_time, y = 1500, label = 'Period:'))
ggsave(filename=here("2 Combined results/Specific/Power_decrease_dyn_test.png"),width=width_graph, height=height_graph)

# --- Animation for video ---
fps = 25
width = 1500
height = 500
timefactor = 3
duration = round(max(df_animate$Elapsed_time)/timefactor,0)

ylim.prim <- c(4.2, 4.6)
ylim.sec <- c(-30, 30)   # for angle
# ylim.sec <- c(-0.8, 0.8) # for acceleration

b <- diff(ylim.prim)/diff(ylim.sec)
a <- ylim.prim[1] - b*ylim.sec[1] 

# Only gas flow
plot1 <- df_animate %>%
  ggplot(aes(x = Elapsed_time, y = Gas_flow)) + 
  geom_line() +
  geom_vline(xintercept = 25, color = default_colors[1]) +
  geom_point(size=3) +
  labs(title = "Fuel inflow", x = 'Time [s]', y = 'Gas flow [SLM]') +
  theme(text = element_text(size=30)) + 
  transition_reveal(Elapsed_time)

# Gas flow and inclination
plot1 <- df_animate %>%
  ggplot(aes(x = Elapsed_time)) + 
  geom_line(aes(y = Gas_flow)) +
  geom_line(aes(y = a + Inclination*b), color = default_colors[2]) +
  geom_vline(xintercept = 25, color = default_colors[1]) +
  geom_point(aes(y = Gas_flow),size=3) +
  labs(title = "Fuel inflow", x = 'Time [s]', y = 'Gas flow [SLM]') +
  scale_y_continuous(sec.axis = sec_axis(~ (. - a)/b, name = "Inclination")) +
  theme(text = element_text(size=30)) + 
  transition_reveal(Elapsed_time)+
  theme(axis.text.y.right = element_text(color = "red"), 
        axis.title.y.right = element_text(color = "red")
        )
# # Gas flow and acceleration
# plot1 <- df_animate %>%
#   ggplot(aes(x = Elapsed_time)) + 
#   geom_line(aes(y = Gas_flow)) +
#   geom_line(aes(y = a + Acceleration_t*b), color = default_colors[2]) +
#   geom_vline(xintercept = 25, color = default_colors[1]) +
#   geom_point(aes(y = Gas_flow),size=3) +
#   labs(title = "Fuel inflow", x = 'Time [s]', y = 'Gas flow [SLM]') +
#   scale_y_continuous(sec.axis = sec_axis(~ (. - a)/b, name = "Acceleration [m/s^2]")) +
#   theme(text = element_text(size=30)) + 
#   transition_reveal(Elapsed_time)+
#   theme(axis.text.y.right = element_text(color = "red"), 
#         axis.title.y.right = element_text(color = "red")
#   )

plot2 <- df_animate %>%
  ggplot(aes(x = Elapsed_time, y = Stack_V)) + 
  geom_line() +
  geom_vline(xintercept = 25, color = default_colors[1]) +
  geom_point(size=3) +
  labs(title = "Stack Voltage", x = 'Time [s]', y = 'Voltage [V]') +
  theme(text = element_text(size=30)) + 
  transition_reveal(Elapsed_time)

plot3 <- df_animate %>%
  ggplot(aes(x = Elapsed_time, y = AC_Power)) + 
  geom_line() +
  geom_vline(xintercept = 25, color = default_colors[1]) +
  geom_point(size=3) +
  labs(title = "AC power", x = 'Time [s]', y = 'Power [W]') +
  theme(text = element_text(size=30)) + 
  transition_reveal(Elapsed_time)

plot4 <- df_animate %>%
  ggplot(aes(x = Elapsed_time, y = Elec_eff_net)) + 
  geom_line() +
  geom_vline(xintercept = 25, color = default_colors[1]) +
  geom_point(size=3) +
  labs(title = "Efficiency", x = 'Time [s]', y = 'Net Electric efficiency [%]') +
  theme(text = element_text(size=30)) + 
  transition_reveal(Elapsed_time)

plot5 <- df_animate %>%
  ggplot(aes(x = Elapsed_time, y = Inclination)) + 
  geom_line() +
  geom_vline(xintercept = 25, color = default_colors[1]) +
  geom_point(size=3) +
  labs(title = "Instantenous inclination angle [°]", x = 'Time [s]', y = 'Inclination') +
  theme(text = element_text(size=30)) + 
  transition_reveal(Elapsed_time)

# R needs to be run as administrator
gif1 <- animate(plot1, duration = duration, fps = fps, width = width, height = height, renderer = gifski_renderer())
anim_save(animation = gif1, filename=here("2 Combined results/Animation",paste("gif1inc_tf",timefactor,".gif",sep="")))
gif2 <- animate(plot2, duration = duration, fps = fps, width = width, height = height, renderer = gifski_renderer())
anim_save(animation = gif2, filename=here("2 Combined results/Animation",paste("gif2_tf",timefactor,".gif",sep="")))
gif3 <- animate(plot3, duration = duration, fps = fps, width = width, height = height, renderer = gifski_renderer())
anim_save(animation = gif3, filename=here("2 Combined results/Animation",paste("gif3_tf",timefactor,".gif",sep="")))
gif4 <- animate(plot4, duration = duration, fps = fps, width = width, height = height, renderer = gifski_renderer())
anim_save(animation = gif4, filename=here("2 Combined results/Animation",paste("gif4_tf",timefactor,".gif",sep="")))
gif5 <- animate(plot5, duration = duration, fps = fps, width = width, height = height, renderer = gifski_renderer())
anim_save(animation = gif5, filename=here("2 Combined results/Animation",paste("gif5_tf",timefactor,".gif",sep="")))


rm(gif1,gif2, gif3, gif4, gif5, fps, combined, myAnimation)
rm(plot1,plot2,plot3,plot4, plot4)

# myAnimation <- image_append(c(gif1[1],gif2[1]))
# 
# for(i in 2:nrow(df_animate)*25){
#   combined <- image_append(c(gif1[i],gif2[i]))
#   myAnimation <- c(myAnimation, combined)
# }
# 
# myAnimation <- animate(myAnimation, duration = 15, fps = 25, width = 200, height = 200, renderer = magick_renderer())

# anim_save(animation = myAnimation, "2 Combined results/Animation/Combined.gif")




